@@ -1,10 +1,11 @@
-from __future__ import absolute_import
-
 from mercurial.i18n import _
 from mercurial import (
     hg,
     registrar,
 )
+from mercurial.utils import (
+    urlutil,
+)
 
 cmdtable = {}
 command = registrar.command(cmdtable)
@@ -18,10 +19,13 @@ def getflogheads(ui, repo, path):
     Used for testing purpose
     """
 
-    dest = repo.ui.expandpath(b'default')
+    dest = urlutil.get_unique_pull_path_obj(b'getflogheads', ui)
     peer = hg.peer(repo, {}, dest)
 
-    flogheads = peer.x_rfl_getflogheads(path)
+    try:
+        flogheads = peer.x_rfl_getflogheads(path)
+    finally:
+        peer.close()
 
     if flogheads:
         for head in flogheads:
