@@ -1,5 +1,3 @@
-from __future__ import print_function, absolute_import
-
 """Fuzz testing for operations against a Mercurial repository
 
 This uses Hypothesis's stateful testing to generate random repository
@@ -38,8 +36,7 @@ except ImportError:
 
 import binascii
 from contextlib import contextmanager
-import errno
-import pipes
+import shlex
 import shutil
 import silenttestrunner
 import subprocess
@@ -88,9 +85,8 @@ while True:
     try:
         os.close(os.open(savefile, os.O_CREAT | os.O_EXCL | os.O_WRONLY))
         break
-    except OSError as e:
-        if e.errno != errno.EEXIST:
-            raise
+    except FileExistsError:
+        pass
 assert os.path.exists(savefile)
 
 hgrc = os.path.join(".hg", "hgrc")
@@ -271,7 +267,7 @@ class verifyingstatemachine(RuleBasedStateMachine):
         if os.path.exists(path):
             return
         self.log.append(
-            "$ mkdir -p -- %s" % (pipes.quote(os.path.relpath(path)),)
+            "$ mkdir -p -- %s" % (shlex.quote(os.path.relpath(path)),)
         )
         os.makedirs(path)
 
@@ -280,7 +276,7 @@ class verifyingstatemachine(RuleBasedStateMachine):
         if path == ".":
             return
         os.chdir(path)
-        self.log.append("$ cd -- %s" % (pipes.quote(path),))
+        self.log.append("$ cd -- %s" % (shlex.quote(path),))
 
     def hg(self, *args):
         extra_flags = []
@@ -290,7 +286,7 @@ class verifyingstatemachine(RuleBasedStateMachine):
         self.command("hg", *(tuple(extra_flags) + args))
 
     def command(self, *args):
-        self.log.append("$ " + ' '.join(map(pipes.quote, args)))
+        self.log.append("$ " + ' '.join(map(shlex.quote, args)))
         subprocess.check_output(args, stderr=subprocess.STDOUT)
 
     # Section: Set up basic data
@@ -354,12 +350,12 @@ class verifyingstatemachine(RuleBasedStateMachine):
             o.write(content)
         self.log.append(
             (
-                "$ $PYTHON -c 'import binascii; "
+                "$ \"$PYTHON\" -c 'import binascii; "
                 "print(binascii.unhexlify(\"%s\"))' > %s"
             )
             % (
                 binascii.hexlify(content),
-                pipes.quote(path),
+                shlex.quote(path),
             )
         )
 
@@ -409,7 +405,7 @@ class verifyingstatemachine(RuleBasedStateMachine):
         if amend:
             errors.append("cannot amend public changesets")
             command.append("--amend")
-        command.append("-m" + pipes.quote(message))
+        command.append("-m" + shlex.quote(message))
         if secret:
             command.append("--secret")
         if close_branch:
@@ -619,8 +615,8 @@ settings.register_profile(
     settings(
         timeout=-1,
         stateful_step_count=1000,
-        max_examples=10 ** 8,
-        max_iterations=10 ** 8,
+        max_examples=10**8,
+        max_iterations=10**8,
         database=writeonlydatabase(settings.default.database),
     ),
 )
